﻿-------------------------------------------------------------------------------
-- A few "local global" variables for this lua-file:                         --
-------------------------------------------------------------------------------
local TIIT_name    = "Toxyna's 'I interrupted that'";
local TIIT_version = "0.0.12"
local TIIT_date    = "2012/09/28"
local TIIT_color   = "|cffafdfd0";   -- addon-color for the mesages
local L            = select(2, ...); -- For internationalization

local Options;     -- short cut for ToxynasIInterruptedThat_Options

-------------------------------------------------------------------------------
-- Provide default values:                                                   --
-------------------------------------------------------------------------------
local TIIT_Defaults = {
    ---------------------------------------------------------------------------
    -- Default options for the persitant saved variables:                    --
    ---------------------------------------------------------------------------
    ["Options"] = {
        ["MasterOnOff"] = "on",        -- addon is active
        ["version"]     = TIIT_version,
        ["debug"]       = false,
        ["mode"]        = "Emote",     -- addon's output channel is emote
        ["message"]     = L["PERSON interrupts the spell SPELL from MOB!"],
                                       -- message for other channels than emote
        ["announceAll"] = false,           -- announce interrupts from all members
    },
    ---------------------------------------------------------------------------
    -- later on here could follow other default options ...                  --
    ---------------------------------------------------------------------------
};

-------------------------------------------------------------------------------
-- Prints the message in TIIT's color:                                       --
-------------------------------------------------------------------------------
local function c_print(message)
    DEFAULT_CHAT_FRAME:AddMessage(TIIT_color .. message .. "|r");
end

-------------------------------------------------------------------------------
-- Prints the two parts, the first in TIIT_color, the second normal:         --
-------------------------------------------------------------------------------
local function cc_print(part1, part2)
    DEFAULT_CHAT_FRAME:AddMessage(TIIT_color .. part1 .. "|r" .. part2);
end

-------------------------------------------------------------------------------
-- Prints the message:                                                       --
-------------------------------------------------------------------------------
local function n_print(message)
    DEFAULT_CHAT_FRAME:AddMessage(message);
end

-------------------------------------------------------------------------------
-- This function is called, when the addon is loading:                       --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_OnLoad(self)
    ---------------------------------------------------------------------------
    -- Greeting on load or reload:                                           --
    ---------------------------------------------------------------------------
    c_print(TIIT_name .. " -- " .. L["/tiit for more."]);

    ---------------------------------------------------------------------------
    -- Install slash command /tiit:                                          --
    ---------------------------------------------------------------------------
    SlashCmdList["TOXYNAS_I_INTERRUPTED_THAT"] =
            ToxynasIInterruptedThat_SlashCommand;
    SLASH_TOXYNAS_I_INTERRUPTED_THAT1 = "/tiit";
    self:RegisterEvent("VARIABLES_LOADED");

    ---------------------------------------------------------------------------
    -- Enable an interface options addons panel:                             --
    ---------------------------------------------------------------------------
    --ToxynasIInterruptedThat_InitOptionsPanel(self);                            -- TRY THIS !!!
end

-------------------------------------------------------------------------------
-- Enable an interface options addons panel:                                 --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_InitOptionsPanel(self)
    ---------------------------------------------------------------------------
    -- Set the name for the Category for the Panel:                          --
    ---------------------------------------------------------------------------
    optionPanel.name = TIIT_name .. " " .. TIIT_version;

    ---------------------------------------------------------------------------
    -- When the player clicks okay, run this function:                       --
    ---------------------------------------------------------------------------
    optionPanel.okay =
        function (self)
            ToxynasIInterruptedThat_OptionPanel_Close();
        end;

    ---------------------------------------------------------------------------
    -- When the player clicks cancel, run this function:                     --
    ---------------------------------------------------------------------------
    optionPanel.cancel =
        function (self)
            ToxynasIInterruptedThat_Option_CancelOrLoad();
        end;

    ---------------------------------------------------------------------------
    -- When the player  clicks default, run this function:                   --
    ---------------------------------------------------------------------------
    optionPanel.default =
        function (self)
            ToxynasIInterruptedThat_Option_Default();
        end;

    ---------------------------------------------------------------------------
    -- Add the optionPanel to the Interface Options:                         --
    ---------------------------------------------------------------------------
    InterfaceOptions_AddCategory(optionPanel);
end

-------------------------------------------------------------------------------
-- When the player clicks okay on the interface options addons panel:        --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_OptionPanel_Close()
    xpcall(function()
        c_print("ToxynasIInterruptedThat_OptionPanel_Close entered.");           -- REMOVE
        -- your original panel.okay() code goes here
    end, geterrorhandler())
end

-------------------------------------------------------------------------------
-- When the player clicks cancel on the interface options addons panel:      --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_Option_CancelOrLoad()
    xpcall(function()
        c_print("ToxynasIInterruptedThat_Option_CancelOrLoad entered.");         -- REMOVE
        -- your original panel.okay() code goes here
    end, geterrorhandler())
end

-------------------------------------------------------------------------------
-- When the player  clicks default on the interface options addons panel:    --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_Option_Default()
    xpcall(function()
        c_print("ToxynasIInterruptedThat_Option_Default entered.");              -- REMOVE
        ToxynasIInterruptedThat_Options = TIIT_Defaults["Options"];
    end, geterrorhandler())
end

-------------------------------------------------------------------------------
-- Persitance Handling                                                       --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_Loaded(...)
    ---------------------------------------------------------------------------
    -- If there are no variables saved, use the default values:              --
    ---------------------------------------------------------------------------
    if (not ToxynasIInterruptedThat_Options) then
        ToxynasIInterruptedThat_Options = TIIT_Defaults["Options"];
        c_print(TIIT_name .. " " ..
                "Options database not found. Generating default values.");       -- I18N !!!

    ---------------------------------------------------------------------------
    -- If there are variables saved, but they have no version:               --
    ---------------------------------------------------------------------------
    elseif (ToxynasIInterruptedThat_Options["version"] == nil) then
        ToxynasIInterruptedThat_Options = TIIT_Defaults["Options"];
        c_print(TIIT_name .. " " ..
                "Options database has no version!" .. " " ..                     -- I18N !!!
                "Generating a new one, please check your setting!");             -- I18N !!!

    ---------------------------------------------------------------------------
    -- If the saved variables have an earlier addon version, copy the known  --
    -- values and get the rest from the default:                             --
    ---------------------------------------------------------------------------
    elseif (ToxynasIInterruptedThat_Options["version"] < TIIT_version) then
        c_print(TIIT_name .. " " ..
                "Elder Options database version found. Merging options ...");    -- I18N !!!
        local options_temp = TIIT_Defaults["Options"];
        local k, v;
        for k, v in pairs(ToxynasIInterruptedThat_Options) do
            if (TIIT_Defaults["Options"][k]) then
                options_temp[k] = v;
            end
        end
        options_temp["version"] = TIIT_version;
        ToxynasIInterruptedThat_Options = options_temp;
    end

    ---------------------------------------------------------------------------
    -- Save the local variable Options as a short cut for                    --
    -- ToxynasIInterruptedThat_Options:                                      --
    ---------------------------------------------------------------------------
    Options = ToxynasIInterruptedThat_Options;
end

-- kann keine boolschen Ausdrücke ausgeben... verbessern...
-- local function print_the_options()
        -- c_print("---- ToxynasIInterruptedThat_Options: ----");                                    -- REMOVE
        -- local k, v;
        -- for k, v in pairs(ToxynasIInterruptedThat_Options) do
            -- c_print(k .. " -> " .. v);
        -- end
        -- c_print("---- Options: ----");                                    -- REMOVE
        -- local kk, vv;
        -- for kk, vv in pairs(Options) do
            -- c_print(kk .. " -> " .. vv);
        -- end
-- end

-------------------------------------------------------------------------------
-- Reacts at unfiltered combat log events.                                   --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_CombatLogEventUnfiltered(...)
    ---------------------------------------------------------------------------
    -- if the addon is not active : return                                   --
    ---------------------------------------------------------------------------
    if Options["MasterOnOff"] == "off" then
        return;
    end

    ---------------------------------------------------------------------------
    -- get some of the event parameters:                                     --
    ---------------------------------------------------------------------------
    local eventType        = select(2, ...);
    local interruptingUser = select(5, ...);
    local interruptedUser  = select(9, ...);
    local spellID          = select(15, ...);
    local interruptedSpell = GetSpellLink(spellID);
    local interruptSpellId = select(12, ...);

    ---------------------------------------------------------------------------
    -- handel the event SPELL_INTERRUPT:                                     --
    ---------------------------------------------------------------------------
    if eventType == "SPELL_INTERRUPT" then
        if Options["debug"] then
            c_print("eventType        = " .. eventType);
            c_print("interruptingUser = " .. interruptingUser);
            c_print("interruptedUser  = " .. interruptedUser);
            c_print("spellID          = " .. spellID);
            c_print("interruptedSpell = " .. interruptedSpell);
            c_print("interruptSpellId = " .. interruptSpellId);
            c_print("interruptSpell   = " .. GetSpellLink(interruptSpellId));
        end

        -----------------------------------------------------------------------
        -- if someone else interrupts and we don't want to tell: return      --
        -----------------------------------------------------------------------
        if not Options["announceAll"] and
            interruptingUser ~= UnitName("player") then
                return;
        end

        -----------------------------------------------------------------------
        -- if the interrupter is not in our party/raid: return               --
        -----------------------------------------------------------------------
        if UnitInRaid("player")  and not UnitInRaid(interruptingUser)  or
           UnitInParty("player") and not UnitInParty(interruptingUser) then
            return;
        end

        -----------------------------------------------------------------------
        -- handle the 'Emote' mode:
        -----------------------------------------------------------------------
        if Options["mode"] == 'Emote' then
            -------------------------------------------------------------------
            -- the following placeholder can be used in the Emote-Message:   --
            -- #INTERRUPTED_USER#  the user or mob, that was casting and was --
            --                     interrupted by the player                 --
            -- #INTERRUPTED_SPELL# the spell that user or mob was casting    --
            -- #INTERRUPTING_USER# the player's name                         --
            -------------------------------------------------------------------
            local message; -- the message that will be shown as a emote
            local default_message = L['interrupts #INTERRUPTED_USER# '
                                    .. 'on casting #INTERRUPTED_SPELL#.'];

            -------------------------------------------------------------------
            -- the player has interrupted:
            -------------------------------------------------------------------
            if interruptingUser == UnitName("player") then
                local localizedClass, englishClass = UnitClass("player");
                if "SHAMAN" == englishClass then
                    message = L['sends a burst of wind to #INTERRUPTED_USER# '
                            .. 'and interrupts #INTERRUPTED_SPELL#.'];
                elseif "ROGUE" == englishClass then
                    -- Rogues can also interrupt via Gauge, Solar Plexus,
                    -- Cheap Shot, or Garotte, but only Kick is a
                    -- "SPELL_INTERRUPT" and I yet have no idea, how to get
                    -- the other four abilities.
                    message = L['kicks #INTERRUPTED_USER# '
                            .. 'and interrupts #INTERRUPTED_SPELL#.'];
                elseif "MAGE" == englishClass then
                    message = L['speaks a counterspell at #INTERRUPTED_USER# '
                            .. 'and interrupts #INTERRUPTED_SPELL#.'];
                elseif "WARRIOR" == englishClass then
                    -- Warriors can interrupt spells via "Pummel"
                    -- (Spell id 6552) and "Disrupting Shout" (Spell id
                    -- 102060), so I provide two different emotes for them:
                    if interruptSpellId == 6552 then -- Pummel
                        message = L['pummels #INTERRUPTED_USER# '
                                .. 'and interrupts #INTERRUPTED_SPELL#.'];
                    elseif interruptSpellId == 102060 then -- Disrupting Shout
                        message = L['shouts loud at #INTERRUPTED_USER# '
                                .. 'and interrupts #INTERRUPTED_SPELL#.'];
                    else
                        message = default_message;
                    end
                    -- Paladine können mit Schild des Rächers unterbrechen (SpellID=31935)
                else
                    message = default_message;
                end

            -------------------------------------------------------------------
            -- someone else has interrupted:
            -------------------------------------------------------------------
            else
                message = L['admired, how #INTERRUPTED_USER# '
                        .. 'is interrupting #INTERRUPTED_USER# on casting '
                        .. '#INTERRUPTED_SPELL#.'];
            end

            -------------------------------------------------------------------
            -- replace the placeholders:
            -------------------------------------------------------------------
            if nil ~= string.find(message, '#INTERRUPTED_USER#', 1, true) then
                message = string.gsub(message, '#INTERRUPTED_USER#', interruptedUser);
            end
            if nil ~= string.find(message, '#INTERRUPTED_SPELL#', 1, true) then
                message = string.gsub(message, '#INTERRUPTED_SPELL#', interruptedSpell);
            end
            if nil ~= string.find(message, '#INTERRUPTING_USER#', 1, true) then
                message = string.gsub(message, '#INTERRUPTING_USER#', interruptingUser);
            end

            -------------------------------------------------------------------
            -- send the mesage:
            -------------------------------------------------------------------
            SendChatMessage(message, "EMOTE");

        -----------------------------------------------------------------------
        -- other channel than 'Emote':
        -----------------------------------------------------------------------
        else
            local numberOfGroupMembers = GetNumGroupMembers();

            -------------------------------------------------------------------
            -- determine the channel for the mode 'Auto':
            -------------------------------------------------------------------
            local autoChannel = "Self";
            if numberOfGroupMembers > 0 and numberOfGroupMembers <= 5 then
                autoChannel = "PARTY";
            elseif numberOfGroupMembers > 5 then
                local ssInInstance, ssinstanceType = IsInInstance();
                -- http://www.wowwiki.com/API_IsInInstance
                -- results: none, pvp, arena, party, raid
                if ssinstanceType == "pvp" then
                    autoChannel = "BATTLEGROUND";
                else
                    autoChannel = "RAID"
                end
            end

            -------------------------------------------------------------------
            -- determine output string:                                      --
            -------------------------------------------------------------------
            local interruptMessage = Options["message"];
            interruptMessage = string.gsub(
                    interruptMessage, 'MOB', interruptedUser);
            interruptMessage = string.gsub(
                    interruptMessage, 'SPELL', GetSpellLink(spellID));
            if interruptingUser ~= UnitName("player") then
                interruptMessage = string.gsub(
                        interruptMessage, 'PERSON', interruptingUser);
            else
                interruptMessage = string.gsub(
                        interruptMessage, 'PERSON', UnitName("player"));
            end

            -------------------------------------------------------------------
            -- Message output depending on the mord:                         --
            -------------------------------------------------------------------
            if Options["mode"] == 'Self' or autoChannel == 'Self' then
                n_print(interruptMessage);
            elseif Options["mode"] == 'Say' then
                if autoChannel == "pvp" then
                    n_print(interruptMessage);
                else
                    SendChatMessage(interruptMessage, "SAY");
                end
            elseif Options["mode"] == 'Auto' then
                if (not inParty) and (not inRaid) then
                    n_print(interruptMessage);
                else
                    SendChatMessage(interruptMessage, autoChannel);
                end
            end
        end
    end
end

-------------------------------------------------------------------------------
-- function TIIT_help() shows the addon's options to the user.               --
-------------------------------------------------------------------------------
local function TIIT_help()
    c_print(TIIT_name .. ' ' .. L['Version'] .. ' ' .. TIIT_version .. ' ' ..
            L['from'] .. ' ' .. TIIT_date ..'|r');
    c_print(L["To switch the interrupt message to a channel "
            .. "use /tiit with say, self, auto, or emote. Other commands are:"]);
    c_print(L["/tiit or /tiit help - show this message"]);
    c_print(L["/tiit info - show detailed informations"]);
    c_print(L["/tiit toggle - switches"] .. " " .. TIIT_name .. " " ..
            L["on or off"]);
    c_print(L["/tiit allmembers - switch interrupt reports from all "
            .. "members on or off."]);
    c_print(L["/tiit msg - show the message (not used for the "
            .. "'Emote'-mode)"]);
    c_print(L["/tiit setmsg - change the message. Use 'MOB' for the "
            .. "enemy's name, 'SPELL' for the enemy's spell you have "
            .. "interrupted, and 'PERSON' for the interrupting Person."]);
end

-------------------------------------------------------------------------------
-- The function TIIT_announce_output_channel() tells the output channel to   --
-- the user.                                                                 --
-------------------------------------------------------------------------------
local function TIIT_announce_output_channel()
    cc_print(L["Output channel is"] .. ": ", Options["mode"]);
end

-------------------------------------------------------------------------------
-- The function TIIT_show_info() show's some state of the addon to the user. --
-------------------------------------------------------------------------------
local function TIIT_show_info()
    if Options["MasterOnOff"] == "of" then
        c_print(TIIT_name .. " is not active.");
    else
        c_print(TIIT_name .. " " .. L["Informations"] .. ":");
        -----------------------------------------------------------------------
        -- mode:
        -----------------------------------------------------------------------
        TIIT_announce_output_channel();

        -----------------------------------------------------------------------
        -- announceAll:
        -----------------------------------------------------------------------
        local onOrOff;
        if Options["announceAll"] then
            onOrOff = L["On"];
        else
            onOrOff = L["Off"];
        end
        cc_print(L["Announce interrupts from all party members"] .. ": ",
                onOrOff);

        -----------------------------------------------------------------------
        -- message:
        -----------------------------------------------------------------------
        cc_print(L['message for modes other than Emote is'] .. ':|r',
                Options["message"]);
    end
end

-------------------------------------------------------------------------------
-- The function TIIT_toggle_addon_active() toggles between activated and     --
-- inactivated state of the addon.                                           --
-------------------------------------------------------------------------------
local function TIIT_toggle_addon_active()
    if Options["MasterOnOff"] == "on" then
        Options["MasterOnOff"] = "off";
        c_print(TIIT_name .. " " .. L["is now switched off."]);
    else
        Options["MasterOnOff"] = "on";
        c_print(TIIT_name .. " " .. L["is now switched on."]);
    end
end

-------------------------------------------------------------------------------
-- The function TIIT_toggle_allmembers() toggles between telling about       --
-- interrupts from other players and not telling so.                         --
-------------------------------------------------------------------------------
local function TIIT_toggle_allmembers()
    if Options["announceAll"] then
        Options["announceAll"] = false
        c_print(TIIT_name .. " - " ..
                L["Show interrupts from all members is switched off."]);
    else
        Options["announceAll"] = true
        c_print(TIIT_name .. " - " ..
                L["Show interrupts from all members is switched on."]);
    end
end

-------------------------------------------------------------------------------
-- function TIIT_set_message() validates the user input and sets the new     --
-- message, if it is validated.                                              --
-------------------------------------------------------------------------------
local function TIIT_set_message(message)
    local has_mob    = nil ~= string.find(message, "MOB",    1, true);
    local has_spell  = nil ~= string.find(message, "SPELL",  1, true);
    local has_person = nil ~= string.find(message, "PERSON", 1, true);
    if has_mob and has_spell and has_person then
        c_print(message .. " " .. L["is valid"]);
        Options["message"] = message;
        cc_print(L["message is set to"] .. ": ", message);
    else
        c_print(message .. " " .. L["is NOT valid (use MOB for the enemy, "
                .. "SPELL for the spell and PERSON for the interrupter "
                .. "as placeholders)"]);
        cc_print(L["message stays"] .. ": ", Options["message"]);
    end
end

-------------------------------------------------------------------------------
-- Shows the message for modes other than emote to the user:                 --
-------------------------------------------------------------------------------
local function TIIT_show_message()
    cc_print(L["the message is"] .. ": ", Options["message"]);
end

-------------------------------------------------------------------------------
-- Reacts on the command /tiit and handels user input starting with /tiit.   --
-------------------------------------------------------------------------------
function ToxynasIInterruptedThat_SlashCommand(command)
    local first_command = gsub(command, "%s*([^%s]+).*", "%1");
    local second_command = gsub(command, "%s*([^%s]+)%s*(.*)", "%2");
    first_command = string.lower(first_command);  -- 'eMotE' -> 'emote'

    if first_command == "" or first_command == "help" then
        TIIT_help();
    elseif first_command == "info" then
        TIIT_show_info();
    elseif first_command == "toggle" then
        TIIT_toggle_addon_active();
    elseif first_command == "say" then
        Options["mode"] = 'Say';
        TIIT_announce_output_channel();
    elseif first_command == "self" then
        Options["mode"] = 'Self';
        TIIT_announce_output_channel();
    elseif first_command == "auto" then
        Options["mode"] = 'Auto';
        TIIT_announce_output_channel();
    elseif first_command == "emote" then
        Options["mode"] = 'Emote';
        TIIT_announce_output_channel();
    elseif (first_command == "allmembers") then
        TIIT_toggle_allmembers();
    elseif (first_command == "msg") then
        TIIT_show_message();
    elseif first_command == 'setmsg' then
        TIIT_set_message(second_command);
    elseif first_command == 'debug' then -- hidden debug option
        Options["debug"] = not Options["debug"];
        if Options["debug"] then
            c_print(TIIT_name .. " debug ist set to true");
        else
            c_print(TIIT_name .. " debug ist set to false");
        end
    else
        n_print(TIIT_color .. TIIT_name .. " " .. L['parameter'] .. " |r" ..
                 first_command .. " " ..
                 TIIT_color .. L['is unknown'] .. ".|r");
    end
end
